/*! \file
\verbatim

Copyright (c) 2004, Sylvain Paris and Francois Sillion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    
    * Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.

    * Neither the name of ARTIS, GRAVIR-IMAG nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

\endverbatim
*
*  This file defines the export function to GNUPLOT.
*/

#ifndef __OUTPUT_TO_GNUPLOT__
#define __OUTPUT_TO_GNUPLOT__

#include <fstream>
#include <iostream>

#include "function_2D_types.h"

#include "msg_stream.h"


namespace Function_2D{

  //! Creates a file that can be viewed with GNUPLOT using
  //! command : splot 'file.gnuplot'
  template <typename Function>
  void output_to_GNUPLOT(const char*     file_name,
			 const Function& f,
			 const real_type x_begin = -16,
			 const real_type x_end = 16,
			 const real_type x_step = 1,
			 const real_type y_begin = -16,
			 const real_type y_end = 16,
			 const real_type y_step = 1);



  
/*
  ###########################################################
  ###########################################################
  ###########################################################
  ##############                               ##############
  ##############  I M P L E M E N T A T I O N  ##############
  ##############                               ##############
  ###########################################################
  ###########################################################
  ###########################################################
*/

  
  
  template <typename Function>
  void output_to_GNUPLOT(const char*     file_name,
			 const Function& f,
			 const real_type x_begin,
			 const real_type x_end,
			 const real_type x_step,
			 const real_type y_begin,
			 const real_type y_end,
			 const real_type y_step){

    using namespace std;
    
    ofstream out(file_name);

    for(real_type x=x_begin;x<=x_end;x+=x_step){
      for(real_type y=y_begin;y<=y_end;y+=y_step){
	out<<x<<' '<<y<<' '<<f(x,y)<<'\n';
      }
      out<<'\n';
    }
    
    out.flush();
    out.close();
  }
  
}

#endif
